package com.tillekesoft.youtubedl;

import android.content.Context;
import android.os.AsyncTask;

import java.util.List;
import java.util.regex.Pattern;

import anywheresoftware.b4a.BA;
import me.harshithgoka.youtubedl.YoutubeDL.Extractor;
import me.harshithgoka.youtubedl.YoutubeDL.Format;
import me.harshithgoka.youtubedl.YoutubeDL.VideoInfo;


@BA.Version(0.25f)
@BA.Author("Github: tastelessjolt, Wrapped by: moster67/Mikael Osterhed")
@BA.DependsOn(values = {"B4AYouTubeDLSupport","kotlin-stdlib-1.3.0","named-regexp-0.2.5","okhttp-3.5.0","okio-1.11.0"})
@BA.Permissions(values = { "android.permission.INTERNET"})
@BA.Events(values = {"OnResult(VideoInfo as Object, StreamInfo as Object)","OnError"})
@BA.ShortName("B4AYouTubeDL")


//Original Github project: https://github.com/tastelessjolt/youtube-dl-android


public class B4aYouTubeDL {
    /**
     * This is a wrap of an Android version of YouTube-DL which will extract a direct link which
     * can be used to download or stream a video.
     * You need to initialize the library, then you pass on the shortened YouTube link (in Google's
     * Share format such as https://youtu.be/IfHG7bj-CE3) to the ExtractLink-method.
     * You will then need to monitor the two available events: OnError and OnResult.
     * OnError will trigger if something goes wrong. The OnResult-event will trigger if everything worked fine
     * and returns two Map-objects. THe first object, VideoInfo, contains general information about the
     * video while the second object, StreamInfo, contains direct-links of the video. The StreamInfo
     * contains different formats of the links and another Map-object which holds the url and other
     * information indicating quality, with or without video/audio etc for each format. Normally the
     * first (0) item in the Map is the best one (quality-wise) but being a Map, the order cannot be guaranteed.
     * You can use the "itag" information to also get the best url. The lowest value of "itag" should be the best one.
     */
    public static void LIBRARY_DOC() {}

    private BA ba;
    private String eventName;
    private Extractor extractor;
    private Pattern youtubeUrlPattern;
    private List<Format> formats;

    /**
     * Initializes the library. Make sure to give an event name too so you
     * can check the events OnError and OnResult.
     */
    public void Initialize(final BA ba, String EventName) {
        this.eventName = EventName.toLowerCase(BA.cul);
        this.ba = ba;
        extractor = new Extractor();
        youtubeUrlPattern = Pattern.compile(extractor._VALID_URL);
    }


    private String preprocess (String s) {
        int index = s.lastIndexOf("#");
        if (index > 0) {
            s = s.substring(0, index);
        }

        s = s.replaceFirst("m.youtube.com", "www.youtube.com");
        s = s.replaceFirst("&.*", "");

        return s;
    }

    /**
     * Extracts from YouTube the direct links which can be used for downloading/streaming.
     * Pass the url using YouTube's share format (for example: https://youtu.be/IfHG7bj-CE3)
     * Check the OnError- and OnResult-events for results.
     */
    public void ExtractLink(String url) {
        url = preprocess(url);

        java.util.regex.Matcher m = youtubeUrlPattern.matcher(url);

        if (!m.find()) {
            BA.Log("Invalid Youtube URL");
            ba.raiseEvent(B4aYouTubeDL.this, eventName + "_onerror", new Object[0]);
            return;
        }

        AsyncTask<String, Void, VideoInfo> asyncTask = new YoutubeDLAsyncTask(ba.context.getApplicationContext(), extractor);
        asyncTask.execute(url);
    }


    //Perhaps making the class static
    class YoutubeDLAsyncTask extends AsyncTask<String, Void, VideoInfo> {
        Context context;
        Extractor ytextractor;

        public YoutubeDLAsyncTask(Context context, Extractor extractor) {
            this.context = context;
            ytextractor = extractor;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected VideoInfo doInBackground(String... strings) {
            String you_url = strings[0];
            return ytextractor.getFormats(you_url);
        }

        @Override
        protected void onPostExecute(VideoInfo videoInfo) {
            if (videoInfo != null) {

                List<Format> formats = videoInfo.getFormats();

                anywheresoftware.b4a.objects.collections.Map.MyMap info = new anywheresoftware.b4a.objects.collections.Map.MyMap();
                info.put("title", videoInfo.getTitle());
                info.put("author", videoInfo.getAuthor());
                info.put("thumbnailUrl", videoInfo.getThumbnail_url());
                info.put("lastUpdated", videoInfo.getLast_updated());
                info.put("length", videoInfo.getLength());
                info.put("videoId", videoInfo.getVideo_id());
                info.put("viewCount", videoInfo.getView_count());

                if (formats.size() > 0) {
                    anywheresoftware.b4a.objects.collections.Map.MyMap urlInfo = new anywheresoftware.b4a.objects.collections.Map.MyMap();
                    for (int i = 0; i<formats.size();i++){
                        anywheresoftware.b4a.objects.collections.Map.MyMap streamInfo = new anywheresoftware.b4a.objects.collections.Map.MyMap();
                        streamInfo.put("quality",formats.get(i).getQuality());
                        streamInfo.put("url",formats.get(i).getUrl());
                        streamInfo.put("downloadState",formats.get(i).getDowmloadState());
                        streamInfo.put("sanitizedFilename",formats.get(i).sanitizeFilename());
                        streamInfo.put("description",formats.get(i).getDescription());
                        streamInfo.put("extension",formats.get(i).getExtension());
                        streamInfo.put("itag",formats.get(i).getItag());
                        streamInfo.put("audio",formats.get(i).getAudio());
                        streamInfo.put("video",formats.get(i).getVideo());
                        streamInfo.put("content",formats.get(i).getContent());
                        streamInfo.put("location",formats.get(i).getLocation());
                        streamInfo.put("type",formats.get(i).getType());
                        urlInfo.put("Url" + i,streamInfo);
                    }

                    ba.raiseEvent(B4aYouTubeDL.this, eventName + "_onresult", new Object[] { info, urlInfo });

                    String finalurl = formats.get(0).getUrl(); //Best quality link
//                    BA.Log(finalurl);

                }
                else {
                    BA.Log("No. of formats: 0 - Not yet implemented encrypted signature");
                    ba.raiseEvent(B4aYouTubeDL.this, eventName + "_onerror", new Object[0]);
                }
            }
            else {
                BA.Log("Error connecting to the Internet");
            }
        }
    };

} //end of class
